package game.renderer.entity;

import java.util.List;

import game.WCF;
import game.entity.npc.EntityNPC;
import game.entity.types.EntityLiving;
import game.init.Items;
import game.item.Item;
import game.item.ItemStack;
import game.model.IBakedModel;
import game.model.ModelManager;
import game.renderer.DefaultVertexFormats;
import game.renderer.GlState;
import game.renderer.ItemModelMesher;
import game.renderer.RenderBuffer;
import game.renderer.Tessellator;
import game.renderer.blockmodel.BakedQuad;
import game.renderer.blockmodel.Transform;
import game.renderer.blockmodel.Transforms;
import game.renderer.texture.TextureManager;
import game.renderer.texture.TextureMap;
import game.renderer.tileentity.TileEntityItemStackRenderer;
import game.world.Facing;
import game.world.Vec3i;

public class RenderItem
{
    private static final String RES_ITEM_GLINT = "textures/glint.png";

    /** False when the renderer is rendering the item's effects into a GUI */
//    private boolean notRenderingEffectsInGUI = true;

    /** Defines the zLevel of rendering of item on GUI. */
    public float zLevel;
    private final ItemModelMesher itemModelMesher;
    private final TextureManager textureManager;

    public RenderItem(TextureManager textureManager, ModelManager modelManager)
    {
        this.textureManager = textureManager;
        this.itemModelMesher = new ItemModelMesher(modelManager);
//        RenderRegistry.registerItems(this.itemModelMesher);
    }

    /**
     * False when the renderer is rendering the item's effects into a GUI
     *  
     * @param isNot If the renderer is not rendering the effects in a GUI
     */
//    public void isNotRenderingEffectsInGUI(boolean isNot)
//    {
//        this.notRenderingEffectsInGUI = isNot;
//    }

    public ItemModelMesher getItemModelMesher()
    {
        return this.itemModelMesher;
    }

    private void renderModel(IBakedModel model, ItemStack stack)
    {
        this.renderModel(model, -1, stack);
    }

    private void renderModel(IBakedModel model, int color)
    {
        this.renderModel(model, color, (ItemStack)null);
    }

    private void renderModel(IBakedModel model, int color, ItemStack stack)
    {
//        Tessellator tessellator = Tessellator.getInstance();
        RenderBuffer worldrenderer = Tessellator.getBuffer();
        worldrenderer.begin(7, DefaultVertexFormats.ITEM);

        for (Facing enumfacing : Facing.values())
        {
            this.renderQuads(worldrenderer, model.getFaceQuads(enumfacing), color, stack);
        }

        this.renderQuads(worldrenderer, model.getGeneralQuads(), color, stack);
        Tessellator.draw();
    }

    public void renderItem(ItemStack stack, IBakedModel model)
    {
        if (stack != null)
        {
            WCF.glPushMatrix();
            WCF.glScalef(0.5F, 0.5F, 0.5F);

            if (model.isBuiltInRenderer())
            {
                WCF.glRotatef(180.0F, 0.0F, 1.0F, 0.0F);
                WCF.glTranslatef(-0.5F, -0.5F, -0.5F);
                GlState.color(1.0F, 1.0F, 1.0F, 1.0F);
                GlState.enableRescaleNormal();
                TileEntityItemStackRenderer.instance.renderByItem(stack);
            }
            else
            {
	            WCF.glTranslatef(-0.5F, -0.5F, -0.5F);
	            this.renderModel(model, stack);
	
	            if (stack.hasEffect())
	            {
	                this.renderEffect(model);
	            }
            }

            WCF.glPopMatrix();
        }
    }

    private void renderEffect(IBakedModel model)
    {
        GlState.depthMask(false);
        GlState.depthFunc(514);
        GlState.disableLighting();
        GlState.blendFunc(768, 1);
        this.textureManager.bindTexture(RES_ITEM_GLINT);
        WCF.glMatrixMode(5890);
        WCF.glPushMatrix();
        WCF.glScalef(8.0F, 8.0F, 8.0F);
        float f = (float)(System.currentTimeMillis() % 3000L) / 3000.0F / 8.0F;
        WCF.glTranslatef(f, 0.0F, 0.0F);
        WCF.glRotatef(-50.0F, 0.0F, 0.0F, 1.0F);
        this.renderModel(model, -8372020);
        WCF.glPopMatrix();
        WCF.glPushMatrix();
        WCF.glScalef(8.0F, 8.0F, 8.0F);
        float f1 = (float)(System.currentTimeMillis() % 4873L) / 4873.0F / 8.0F;
        WCF.glTranslatef(-f1, 0.0F, 0.0F);
        WCF.glRotatef(10.0F, 0.0F, 0.0F, 1.0F);
        this.renderModel(model, -8372020);
        WCF.glPopMatrix();
        WCF.glMatrixMode(5888);
        GlState.blendFunc(770, 771);
        GlState.enableLighting();
        GlState.depthFunc(515);
        GlState.depthMask(true);
        this.textureManager.bindTexture(TextureMap.locationBlocksTexture);
    }

    private void putQuadNormal(RenderBuffer renderer, BakedQuad quad)
    {
        Vec3i vec3i = quad.getFace().getDirectionVec();
        renderer.putNormal((float)vec3i.getX(), (float)vec3i.getY(), (float)vec3i.getZ());
    }

    private void renderQuad(RenderBuffer renderer, BakedQuad quad, int color)
    {
        renderer.addVertexData(quad.getVertexData());
        renderer.putColor4(color);
        this.putQuadNormal(renderer, quad);
    }

    private void renderQuads(RenderBuffer renderer, List<BakedQuad> quads, int color, ItemStack stack)
    {
        boolean flag = color == -1 && stack != null;
        int i = 0;

        for (int j = quads.size(); i < j; ++i)
        {
            BakedQuad bakedquad = (BakedQuad)quads.get(i);
            int k = color;

            if (flag && bakedquad.hasTintIndex())
            {
                k = stack.getItem().getColorFromItemStack(stack, bakedquad.getTintIndex());

//                if (EntityRenderer.anaglyphEnable)
//                {
//                    k = TextureUtil.anaglyphColor(k);
//                }

                k = k | -16777216;
            }

            this.renderQuad(renderer, bakedquad, k);
        }
    }

    public boolean shouldRenderItemIn3D(ItemStack stack)
    {
        IBakedModel ibakedmodel = this.itemModelMesher.getItemModel(stack);
        return ibakedmodel == null ? false : ibakedmodel.isGui3d();
    }

    private void preTransform(ItemStack stack)
    {
        IBakedModel ibakedmodel = this.itemModelMesher.getItemModel(stack);
        Item item = stack.getItem();

        if (item != null)
        {
            boolean flag = ibakedmodel.isGui3d();

            if (!flag)
            {
                WCF.glScalef(2.0F, 2.0F, 2.0F);
            }

            GlState.color(1.0F, 1.0F, 1.0F, 1.0F);
        }
    }

    public void renderItem(ItemStack stack, Transforms.Camera cameraTransformType)
    {
        if (stack != null)
        {
            IBakedModel ibakedmodel = this.itemModelMesher.getItemModel(stack);
            this.renderItemModelTransform(stack, ibakedmodel, cameraTransformType);
        }
    }

    public void renderItemModelForEntity(ItemStack stack, EntityLiving entityToRenderFor, Transforms.Camera cameraTransformType)
    {
        if (stack != null && entityToRenderFor != null)
        {
            IBakedModel ibakedmodel = this.itemModelMesher.getItemModel(stack);

            if (entityToRenderFor.isPlayer())
            {
                EntityNPC entityplayer = (EntityNPC)entityToRenderFor;
                Item item = stack.getItem();
                String modelresourcelocation = null;

                if (item == Items.fishing_rod && entityplayer.fishEntity != null)
                {
                    modelresourcelocation = "item/fishing_rod#1" + '#' + "inventory";
                }
                else if (item == Items.bow && entityplayer.getItemInUse() != null)
                {
                    int i = stack.getMaxItemUseDuration() - entityplayer.getItemInUseCount();

                    if (i >= 18)
                    {
                        modelresourcelocation = "item/bow#3" + '#' + "inventory";
                    }
                    else if (i > 13)
                    {
                        modelresourcelocation = "item/bow#2" + '#' + "inventory";
                    }
                    else if (i > 0)
                    {
                        modelresourcelocation = "item/bow#1" + '#' + "inventory";
                    }
                }

                if (modelresourcelocation != null)
                {
                    ibakedmodel = this.itemModelMesher.getModelManager().getModel(modelresourcelocation);
                }
            }

            this.renderItemModelTransform(stack, ibakedmodel, cameraTransformType);
        }
    }

    protected void renderItemModelTransform(ItemStack stack, IBakedModel model, Transforms.Camera cameraTransformType)
    {
        this.textureManager.bindTexture(TextureMap.locationBlocksTexture);
//        this.textureManager.getTexture(TextureMap.locationBlocksTexture).unsetMipmap();
        this.preTransform(stack);
        GlState.enableRescaleNormal();
        GlState.alphaFunc(516, 0.1F);
        GlState.enableBlend();
        GlState.tryBlendFuncSeparate(770, 771, 1, 0);
        WCF.glPushMatrix();
        Transforms itemcameratransforms = model.getItemCameraTransforms();
        itemcameratransforms.apply(cameraTransformType);

        if (this.isThereOneNegativeScale(itemcameratransforms.get(cameraTransformType)))
        {
            GlState.cullFace(1028);
        }

        this.renderItem(stack, model);
        GlState.cullFace(1029);
        WCF.glPopMatrix();
        GlState.disableRescaleNormal();
        GlState.disableBlend();
        this.textureManager.bindTexture(TextureMap.locationBlocksTexture);
//        this.textureManager.getTexture(TextureMap.locationBlocksTexture).restoreLastMipmap();
    }

    /**
     * Return true if only one scale is negative
     *  
     * @param itemTranformVec The ItemTransformVec3f instance
     */
    private boolean isThereOneNegativeScale(Transform itemTranformVec)
    {
        return itemTranformVec.scale.x < 0.0F ^ itemTranformVec.scale.y < 0.0F ^ itemTranformVec.scale.z < 0.0F;
    }

    private void renderItemIntoGUI(ItemStack stack, int x, int y)
    {
        IBakedModel ibakedmodel = this.itemModelMesher.getItemModel(stack);
        WCF.glPushMatrix();
        this.textureManager.bindTexture(TextureMap.locationBlocksTexture);
//        this.textureManager.getTexture(TextureMap.locationBlocksTexture).unsetMipmap();
        GlState.enableRescaleNormal();
        GlState.enableAlpha();
        GlState.alphaFunc(516, 0.1F);
        GlState.enableBlend();
        GlState.blendFunc(770, 771);
        GlState.color(1.0F, 1.0F, 1.0F, 1.0F);
        this.setupGuiTransform(x, y, ibakedmodel.isGui3d());
        ibakedmodel.getItemCameraTransforms().apply(Transforms.Camera.GUI);
        this.renderItem(stack, ibakedmodel);
        GlState.disableAlpha();
        GlState.disableRescaleNormal();
        GlState.disableLighting();
        WCF.glPopMatrix();
        this.textureManager.bindTexture(TextureMap.locationBlocksTexture);
//        this.textureManager.getTexture(TextureMap.locationBlocksTexture).restoreLastMipmap();
    }

    private void setupGuiTransform(int xPosition, int yPosition, boolean isGui3d)
    {
        WCF.glTranslatef((float)xPosition, (float)yPosition, 100.0F + this.zLevel);
        WCF.glTranslatef(8.0F, 8.0F, 0.0F);
        WCF.glScalef(1.0F, 1.0F, -1.0F);
        WCF.glScalef(0.5F, 0.5F, 0.5F);

        if (isGui3d)
        {
            WCF.glScalef(40.0F, 40.0F, 40.0F);
            WCF.glRotatef(210.0F, 1.0F, 0.0F, 0.0F);
            WCF.glRotatef(-135.0F, 0.0F, 1.0F, 0.0F);
            GlState.enableLighting();
        }
        else
        {
            WCF.glScalef(64.0F, 64.0F, 64.0F);
            WCF.glRotatef(180.0F, 1.0F, 0.0F, 0.0F);
            GlState.disableLighting();
        }
    }

    public void renderItemAndEffectIntoGUI(final ItemStack stack, int xPosition, int yPosition)
    {
        if (stack != null && stack.getItem() != null)
        {
            this.zLevel += 50.0F;

            this.renderItemIntoGUI(stack, xPosition, yPosition);

            this.zLevel -= 50.0F;
        }
    }

    public void onReload()
    {
        this.itemModelMesher.rebuildCache();
    }
}
